@@ -2,7 +2,7 @@ package SQL::Executor;
 use parent qw(Exporter);
 use strict;
 use warnings;
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 our @EXPORT_OK = qw(named_bind);
 
@@ -12,9 +12,9 @@ use Class::Accessor::Lite (
 );
 use SQL::Maker;
 use Carp qw();
+use Try::Tiny;
 use SQL::Executor::Iterator;
 
-
 =head1 NAME
 
 SQL::Executor - Thin DBI wrapper using SQL::Maker
@@ -308,6 +308,17 @@ this method returns hash ref and it is the same as return value in DBI's selectr
 =cut
 
 sub select_row_by_sql {
+    my ($self, $sql, $binds_aref, $table_name) = @_;
+    my $row = undef;
+    try {
+        $row = $self->_select_row_by_sql($sql, $binds_aref, $table_name);
+    } catch {
+        $self->_handle_exception($sql, $binds_aref, $_);
+    };
+    return $row;
+}
+
+sub _select_row_by_sql {
     my ($self, $sql, $binds_aref, $table_name) = @_;
     my $dbh = $self->dbh;
     my $row = $dbh->selectrow_hashref($sql, undef, @{ $binds_aref || [] } );
@@ -318,6 +329,13 @@ sub select_row_by_sql {
     return $row;
 }
 
+sub _handle_exception {
+    my ($self, $sql, $binds_aref, $err) = @_;
+    my $binds_text = join(',', map{ defined $_ ? $_ : 'NULL' } @{ $binds_aref || [] });
+    my $message = "Error sql: $sql, binds: [$binds_text]\n$_";
+    Carp::croak($message);
+}
+
 =head2 select_all_by_sql($sql, \@binds, $table_name)
 
 select all rows.
@@ -330,6 +348,17 @@ this method returns array that is composed of hash refs. (hash ref is same as DB
 =cut
 
 sub select_all_by_sql {
+    my ($self, $sql, $binds_aref, $table_name) = @_;
+    my @result = ();
+    try {
+        @result = $self->_select_all_by_sql($sql, $binds_aref, $table_name);
+    } catch {
+        $self->_handle_exception($sql, $binds_aref, $_);
+    };
+    return @result;
+}
+
+sub _select_all_by_sql {
     my ($self, $sql, $binds_aref, $table_name) = @_;
     my $dbh = $self->dbh;
     my @rows = @{ $dbh->selectall_arrayref($sql, { Slice => {} }, @{ $binds_aref || [] }) };
@@ -342,7 +371,6 @@ sub select_all_by_sql {
     return @rows;
 }
 
-
 =head2 select_itr_by_sql($sql, \@binds, $table_name)
 
 select and returns iterator
@@ -355,6 +383,17 @@ Iterator is L<SQL::Executor::Iterator> object.
 =cut
 
 sub select_itr_by_sql {
+    my ($self, $sql, $binds_aref, $table_name) = @_;
+    my $itr = undef;
+    try {
+        $itr = $self->_select_itr_by_sql($sql, $binds_aref, $table_name);
+    } catch {
+        $self->_handle_exception($sql, $binds_aref, $_);
+    };
+    return $itr;
+}
+
+sub _select_itr_by_sql {
     my ($self, $sql, $binds_aref, $table_name) = @_;
     my $dbh = $self->dbh;
     my $sth = $dbh->prepare($sql);
@@ -495,6 +534,16 @@ execute query and returns statement handler($sth).
 =cut
 
 sub execute_query {
+    my ($self, $sql, $binds_aref) = @_;
+    my $sth = undef;
+    try {
+        $self->_execute_query($sql, $binds_aref);
+    } catch {
+        $self->_handle_exception($sql, $binds_aref, $_);
+    };
+}
+
+sub _execute_query {
     my ($self, $sql, $binds_aref) = @_;
     my $dbh = $self->dbh;
     my $sth = $dbh->prepare($sql);
@@ -510,15 +559,21 @@ execute query with named placeholder and returns statement handler($sth).
 
 sub execute_query_named {
     my ($self, $sql, $params_href) = @_;
-    my $dbh = $self->dbh;
+    my $sth = undef;
     my ($new_sql, @binds) = named_bind($sql, $params_href, $self->check_empty_bind);
-    my $sth = $dbh->prepare($new_sql);
-    $sth->execute(@binds);
+    try {
+        my $dbh = $self->dbh;
+        $sth = $dbh->prepare($new_sql);
+        $sth->execute(@binds);
+    } catch {
+        $self->_handle_exception($new_sql, \@binds, $_);
+    };
     return $sth;
 }
 
 
 
+
 =head2 disable_callback()
 
 disable callback temporarily, 
